#!/bin/sh

# This script adds a new virtual server from the command line
# It's designed for ISPs who might need to create a lot of servers

usage()
{
  echo
  echo "Adds a new virtual server"
  echo Usage: $0 '<name> <docroot> <ipname> <port> <--clone template>'
  echo
  echo "Please note that the order in which arguments are supplied is important."
  exit
}


# check ZEUSHOME is set
if [ -z "$ZEUSHOME" ]
then
  echo "Please set \$ZEUSHOME to point to the directory under which Zeus applications are installed"
  exit 1
fi


# Note we still allow five arguments as we used to take a password up to 
# Zeus 3.1.x, and people may have provisioning scripts that pass it
#
# Because of this, in order to add the template functionality we have
# to specifically ask for it with --clone (so we can ignore arguments
# when required).
if [ $# -ne 4  -a $# -ne 5 -a $# -ne 6 ]
then
  usage
else
  # Do we have 6 arguments?
  if [ $# -eq 6 ]
  then
    if [ "$5" != "--clone" ] # If yes, then $5 must be --clone
    then
      usage
    else
      # We're being supplied the template file - can we find it?
      if [ -f "$ZEUSHOME/webadmin/conf/$6" ]
      then
        # Yes
        TEMPLATE=$6
      else
        # No
        echo "Could not find template \"$6\" in $ZEUSHOME/webadmin/conf."
	exit 1
      fi
    fi
  else # We don't have 6 arguments
    if [ "$5" = "--clone" ]
    then # The user has probably made a mistake and not specified the template to use
      usage
    else
    TEMPLATE="skel"
    fi
  fi
fi

# Take the arguments
NAME=$1
DOCROOT=$2
IPNAME=$3
PORT=$4  

# not used, but here to stop people using it for something else
PASSWD=$5

if [ ! -d $ZEUSHOME/webadmin ]
then
    echo "This script should only be run on machines with the webadmin package on"
    exit 1
fi

PATH=$ZEUSHOME/webadmin/bin:$PATH
cd $ZEUSHOME

CONFFILE=$ZEUSHOME/webadmin/conf/sites/$NAME
CONFDIR=$ZEUSHOME/webadmin/conf/sites/
if [ -f $CONFFILE ]
then
  echo "Server $NAME already exists"
  exit 1
fi

# Copy across the defaults from the template file
if [ -w $CONFDIR ]
then
   grep -v '^port ' $ZEUSHOME/webadmin/conf/$TEMPLATE | \
    grep -v '^docroot ' | grep -v '^ip_name ' > $CONFFILE
else
   echo "Cannot write to $CONFFILE: Permission denied"
   exit 1
fi

# add the settings elided by grep -v
echo "docroot $DOCROOT" >> $CONFFILE
echo "ip_name $IPNAME"  >> $CONFFILE
echo "port $PORT"       >> $CONFFILE
